package core.services;

import core.api.models.timemanager.tasks.repeatinginfo.DeadlineStrategy;
import core.api.models.timemanager.tasks.repeatinginfo.TaskRepeatDayInfo;
import core.api.models.timemanager.tasks.repeatinginfo.TaskRepeatWeekInfo;
import core.entities.timemanager.Task;
import core.entities.timemanager.TaskSerie;
import core.repositories.timemanager.TaskRepository;
import core.repositories.timemanager.TaskSeriesRepository;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.time.LocalDate;

@Service
public class TaskSeriesService {

    @Autowired private TaskRepository taskRepository;
    @Autowired private TaskSeriesRepository taskSeriesRepository;

    public ServiceExitCode createTaskSeries(Task rootTask, TaskRepeatWeekInfo taskRepeatInfo) {
        return ServiceExitCode.OK;
    }

    public ServiceExitCode createTaskSeries(Task rootTask, TaskRepeatDayInfo taskRepeatInfo) {
        if(taskRepeatInfo.getDeadlineStrategy() == DeadlineStrategy.FIX_DEADLINE) {
            return ServiceExitCode.INVALID_PARAMETER;
        }

        LocalDate currentTaskDate = rootTask.getStartDate().plusDays(taskRepeatInfo.getOffset());
        TaskSerie taskSerie = new TaskSerie();
        int index = 1;
        while(currentTaskDate.isBefore(taskRepeatInfo.getEndDate())) {
            Task task = Task.cloneTask(rootTask);
            task.setStartDate(currentTaskDate);
            if(taskRepeatInfo.getDeadlineStrategy() == DeadlineStrategy.DEADLINE_EQUAL_START) {
                task.setDeadline(currentTaskDate);
            } else if(taskRepeatInfo.getDeadlineStrategy() == DeadlineStrategy.DEADLINE_FIT_START) {
                task.setDeadline(currentTaskDate.plusDays(taskRepeatInfo.getOffset()-1));
            }
            taskSerie.addTask(task);
            task.setTaskSerie(taskSerie);
            task.setTaskName(task.getTaskName().replace("${i}", convertIndexToString(index)));
            currentTaskDate = currentTaskDate.plusDays(taskRepeatInfo.getOffset());
            index++;
        }

        taskRepository.saveAll(taskSerie.getTasks());
        return ServiceExitCode.OK;
    }

    public static String convertIndexToString(int index) {
        if(index < 10) {
            return "0" + index;
        } else {
            return String.valueOf(index);
        }
    }
}
